<?php
/**
 * AMD Telecom
 *
 * DISCLAIMER
 *
 * AMD Telecom grants you a revocable, non-exclusive, non-transferable, limited
 * license to download, install and use the Router Application solely for your commercial
 * purposes strictly in accordance with the terms of this Agreement.
 *
 * @author AMD Telecom <products@amdtelecom.net>
 * @copyright  AMD Telecom SA
 * @license https://go.routee.net
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Routee extends Module
{

    /**
     * @var string
     */
    private $loginUrl = 'https://api.waymore.io/api/authenticate';
    /**
     * @var string
     */
    private $dataUrl  = 'https://idata.routee.net/api/data';
    /**
     * @var string
     */
    private $eventUrl = 'https://idata.routee.net/api/event';

    /**
     *
     */
    const DEFAULT_LOG_FILE ="prestashop_system.log";

    /**
     *
     */
    public function __construct()
    {
        $this->name = 'Routee';
        $this->tab = 'advertising_marketing';
        $this->version = '3.4.3';
        $this->author = 'Routee';
        $this->need_instance = 0;
        $this->description = "Build campaigns, share content, engage clients via all available devices and messaging options. With Routee you can do the right thing at the right time making sure you deliver.";
        $this->bootstrap = true;

        parent::__construct();
    }

    /**
     * On install event
     *
     * @return bool
     */
    public function install()
    {
        if (!parent::install()) {
            return false;
        }

        Configuration::deleteByName('Routee');
        Configuration::deleteByName('routee_username');
        Configuration::deleteByName('routee_password');
        Configuration::deleteByName('uuid');

        // Run sql for creating DB tables
        Db::getInstance()->execute('CREATE TABLE '._DB_PREFIX_.'routee_events(
            id INT UNSIGNED NOT NULL AUTO_INCREMENT,
            customer_id INT(11) UNSIGNED NOT NULL,
            product_id INT(11) UNSIGNED NOT NULL,
            order_id INT(11) UNSIGNED NOT NULL,
            event varchar(255) NOT NULL,
            json longtext NOT NULL,
            PRIMARY KEY (id)
        ) DEFAULT CHARSET=utf8;');

        // Run sql for creating Logs DB table
        Db::getInstance()->execute('CREATE TABLE '._DB_PREFIX_.'store_events_logs (
            `id` bigint(20) UNSIGNED NOT NULL AUTO_INCREMENT,
            `store_url` varchar(256) NOT NULL,
            `log_type` tinyint(4) NOT NULL COMMENT "0 for error, 1 for success",
            `event_type` tinyint(4) NOT NULL COMMENT "1 = auth, 2 = mass data, 3 = events",
            `log_data` json NOT NULL,
            `is_exported` tinyint(4) NOT NULL DEFAULT "0" COMMENT "0 for no, 1 for yes",
            `created_at` varchar(32) NOT NULL,
            PRIMARY KEY (id)
        ) DEFAULT CHARSET=utf8;');

        return true;
    }

    /**
     * On uninstall event
     *
     * @return bool
     */
    public function uninstall()
    {
        if($this->getUUID() != '') {
            $curlData = array(
                "event" => "Uninstall",
                "uuid" => $this->getUUID(),
                "data" => new stdClass()
            );

            Configuration::deleteByName('Routee');
            Configuration::deleteByName('routee_username');
            Configuration::deleteByName('routee_password');
            Configuration::deleteByName('uuid');
            $this->unsetEvents();
            $this->request($this->eventUrl, $curlData, "POST", 'events');
        }

        return parent::uninstall();
    }

    /**
     * Unregister events
     *
     * @return bool
     */
    public function unsetEvents()
    {
        if (!$this->unregisterHook('actionCartSave')) {
            return false;
        }
        if (!$this->unregisterHook('actionCustomerAccountAdd')) {
            return false;
        }
        if (!$this->unregisterHook('actionCustomerAccountUpdate')) {
            return false;
        }
        if (!$this->unregisterHook('actionValidateCustomerAddressForm')) {
            return false;
        }
        if (!$this->unregisterHook('actionOrderStatusPostUpdate')) {
            return false;
        }
        if (!$this->unregisterHook('actionValidateOrder')) {
            return false;
        }
        if (!$this->unregisterHook('actionPaymentConfirmation')) {
            return false;
        }
		if (!$this->unregisterHook('actionProductSave')) {
            return false;
        }
        if (!$this->unregisterHook('actionProductOutOfStock')) {
            return false;
        }
        if (!$this->unregisterHook('actionUpdateQuantity')) {
            return false;
        }
        if (!$this->unregisterHook('actionSearch')) {
            return false;
        }

        /**
         * These hooks added for Prestashop 1.6 varsion
         */
        if(_PS_VERSION_ < '1.6.9') {
            if (!$this->registerHook('actionObjectAddressAddAfter')) {
                return false;
            }

            if (!$this->registerHook('actionObjectAddressUpdateAfter')) {
                return false;
            }

            if (!$this->registerHook('actionObjectCustomerUpdateAfter')) {
                return false;
            }
        }

        return true;
    }

    /**
     * Register events
     *
     * @return bool
     */
    public function setEvents()
    {
        if (!$this->registerHook('actionCartSave')) {
            return false;
        }
        if (!$this->registerHook('actionCustomerAccountAdd')) {
            return false;
        }
        if (!$this->registerHook('actionCustomerAccountUpdate')) {
            return false;
        }
        if (!$this->registerHook('actionValidateCustomerAddressForm')) {
            return false;
        }
        if (!$this->registerHook('actionOrderStatusPostUpdate')) {
            return false;
        }
        if (!$this->registerHook('actionValidateOrder')) {
            return false;
        }
        if (!$this->registerHook('actionPaymentConfirmation')) {
            return false;
        }
		if (!$this->registerHook('actionProductSave')) {
            return false;
        }
        if (!$this->registerHook('actionProductOutOfStock')) {
            return false;
        }
        if (!$this->registerHook('actionUpdateQuantity')) {
            return false;
        }
        if (!$this->registerHook('actionSearch')) {
            return false;
        }

        /**
         * These hooks added for Prestashop 1.6 varsion
         */
        if(_PS_VERSION_ < '1.6.9') {
            if (!$this->registerHook('actionObjectAddressAddAfter')) {
                return false;
            }

            if (!$this->registerHook('actionObjectAddressUpdateAfter')) {
                return false;
            }

            if (!$this->registerHook('actionObjectCustomerUpdateAfter')) {
                return false;
            }
        }

        return true;
    }

    /**
     * Plugin Configuration Form
     */
    public function getContent()
    {
        $output = null;

        if (Tools::isSubmit('submit' . $this->name)) {
            $this->eventExecutedLog('Routee_login', 'auth');

            $username = (string)Tools::getValue('username');
            $password = (string)Tools::getValue('password');

            if(empty($username) || empty($password)) {
                $output .= $this->displayError('Please fill all the fields.');
                return $output . $this->displayForm();
            }

            $this->eventGrabData('Routee_login', ["username" => $username, "password" => '*****'], 'auth');

            $domain = Tools::getHttpHost(true) . __PS_BASE_URI__;

            $data = array();
            $data["username"] = $username;
            $data["password"] = $password;
            $data["domain"] = $domain;
            $data["source"] = "PrestaShop";
            $data["type"] = "cms";
            $data["version"] = _PS_VERSION_;
            $data["plugin_version"] = $this->version;

            if (empty($this->getUUID())) {
                $logPayload = $data;
                $logPayload['password'] = '*****';
                $this->eventPayloadData('Routee_login', $logPayload, 'auth');

                $res = $this->request($this->loginUrl, $data, "POST", 'auth');

                if ($res["success"]) {
                    if ($this->setEvents()) {
                        $uuid = $res["response"]->uuid;
                        $this->conf_post_uuid = $uuid;
						
						Db::getInstance()->execute(
                            "INSERT INTO `" . _DB_PREFIX_ . "configuration` (name, value) VALUES ('uuid', '$uuid'),
                                ('routee_username', '$username'),
                                ('routee_password', '$password');"
                        );

                        $this->setData();
                        Configuration::updateValue('Routee', true);


                        $output .= $this->displayConfirmation($this->l($res["response"]->message));
                   } else {
                       $output .= $this->displayError($this->l(
                           "Routee was unable to register the event hooks due to your shop's configurations."
                        ));
                    }
                } else {
                    $output .= $this->displayError($this->l($res["response"]->message));
          
                }
            } else {
                $output .= $this->displayConfirmation($this->l("You are already authorized."));
            }
        }
        return $output . $this->displayForm();
    }

    /**
     * @return mixed
     */
    public function displayForm()
    {
        // Get default language
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        $username = empty(Tools::getValue('username')) ? Configuration::get('routee_username') : Tools::getValue('username');
        $password = empty(Tools::getValue('password')) ? Configuration::get('routee_password') : Tools::getValue('password');
        $uuid = empty($this->conf_post_uuid) ? $this->getUUID() : $this->conf_post_uuid;

        // Init Fields form array
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                ),
                'input' => array(
                    array(
                        'type' => 'title',
                        'text' => 'Before login you must create an account at ' .
                            '<a href="https://go.Routee.net" target="_blank">go.Routee.net</a>'
                    ),
                    array(
                        'type' => 'username',
                        'label' => $this->l('Username:'),
                        'name' => 'username',
						'value' => (string)$username,
                        'size' => 64,
                        'required' => true
                    ),
                    array(
                        'type' => 'password',
                        'label' => $this->l('Password:'),
                        'name' => 'password',
						'value' => (string)$password,
                        'size' => 64,
                        'required' => true
                    ),
                    array(
                        'type' => 'routee_uuid',
                        'label' => $uuid != '' ? $this->l('UUID:') : '',
                        'name' => 'routee_uuid',
                        'value' => (string)$uuid,
                        'size' => 64,
                        'required' => false
                    ),
                    array(
                        'type' => 'paragraph',
                        'text' => "Use Routee Platform to create marketing automation. </br>" .
                            "Send automated SMS, Emails, Push Notifications, " .
                            "Voice and Viber messages on events of your choice. </br>" .
                            "<a target=\"_blank\" href='https://www.Routee.net/integrations/prestashop'>" .
                            "<strong>Read More</strong></a>"
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Submit'),
                    'class' => 'btn btn-default pull-right'
                )
            )
        );

        $export_logs_btn = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Export Logs')
                ),
                'input' => array(
                    array(
                        'type' => 'log_export_csv',
                        'label' => $this->l('Export Logs CSV'),
                        'desc' => $this->l('Export logs into csv file.'),
                        'name' => 'export_logs_csv',
                    ),
                    array(
                        'type' => 'log_export_api',
                        'label' => $this->l('Export Logs API'),
                        'desc' => $this->l('Export logs to API.'),
                        'name' => 'export_logs_api',
                    )
                )
            ),
        );

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;

        // Language
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit' . $this->name;
        $helper->toolbar_btn = array(
            'save' =>
                array(
                    'desc' => $this->l('Save'),
                    'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                        '&token=' . Tools::getAdminTokenLite('AdminModules'),
                ),
            'back' => array(
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        return $helper->generateForm(array($fields_form, $export_logs_btn));
    }

    /**
     * Send mass data
     *
     * @return bool
     */
    public function setData()
    {
        $this->setEvents();
        $uuid = $this->getUUID();
        $cartUrl = Tools::getHttpHost(true) . __PS_BASE_URI__ . "cart?action=show";

        $offset = 0;
        $limit = 100;
        $flag = true;
        $delayCounter = 0;
        $this->eventExecutedLog('MassCustomer', 'massdata');

        while ($flag) {
            $customers = Db::getInstance()->executeS(
                "SELECT * FROM " . _DB_PREFIX_ . "customer ORDER BY id_customer LIMIT $limit OFFSET $offset"
            );

            $this->eventGrabData('MassCustomer', count($customers), 'massdata');

            $returnData = array();
            if ((is_array($customers) || is_object($customers))) {
                foreach ($customers as $customer) {
                    $id = $customer["id_customer"];

                    //Set customer format
                    $customerParsed = array(
                        "id" => $customer["id_customer"],
                        "lastname" => $customer["lastname"],
                        "firstname" => $customer["firstname"],
                        "birthday" => $customer["birthday"],
                        "email" => $customer["email"],
                        "company" => $customer["company"],
                        "newsletter" => ($customer["newsletter"] == false) ? false : true,
                        "website" => $customer["website"],
                        "date_add" => $customer["date_add"],
                        "date_upd" => $customer["date_upd"]
                    );
                    $addresses = Db::getInstance()->executeS(
                        "SELECT * FROM " . _DB_PREFIX_ . "address WHERE id_customer='$id'"
                    );
                    $addressesParsed = array();

                    //Set addresses format
                    foreach ($addresses as $address) {
                        $addressParsed = array(
                            "lastname" => $address["lastname"],
                            "firstname" => $address["firstname"],
                            "address1" => $address["address1"],
                            "address2" => $address["address2"],
                            "postcode" => $address["postcode"],
                            "phone" => $address["phone"],
                            "phone_mobile" => $address["phone_mobile"],
                            "city" => $address["city"],
                            "country" => $this->getCountryById($address["id_country"]),
                            "company" => $address["company"]
                        );
                        $addressesParsed[] = $addressParsed;
                    }
                    $carts = Db::getInstance()->executeS(
                        "SELECT * FROM " . _DB_PREFIX_ . "cart WHERE id_customer='$id'"
                    );
                    $respCartProducts = array();
                    $respCartDetails = new stdClass();
                    // Fill cart with products
                    if (is_array($carts) || is_object($carts)) {
                        foreach ($carts as $cart) {
                            $cartId = $cart['id_cart'];
                            $orders = Db::getInstance()->executeS(
                                "SELECT * FROM " . _DB_PREFIX_ . "orders WHERE id_cart='$cartId'"
                            );
                            $cart_products = Db::getInstance()->executeS(
                                "SELECT * FROM " . _DB_PREFIX_ . "cart_product WHERE id_cart='$cartId'"
                            );
                            if (!(is_array($orders) && sizeof($orders) > 0)) {
                                if (is_array($cart_products) && sizeof($cart_products) > 0) {
                                    $respCartDetails->date_add = $cart["date_add"];
                                    $respCartDetails->date_upd = $cart["date_upd"];

                                    foreach ($cart_products as $cart_product) {
                                        $temp_cart_product = array();
                                        $temp_cart_product["product_id"] = $cart_product->id_product;
                                        $temp_cart_product["option_id"] = $cart_product->id_product_attribute;
                                        $temp_cart_product["quantity"] = $cart_product->quantity;
                                        $temp_cart_product["quantity"] = $cart_product->quantity;
                                        $temp_cart_product["date_add"] = $cart_product->date_add;
                                        $respCartProducts[] = $temp_cart_product;
                                    }
                                }
                            }
                        }
                    }

                    $tempReturnObject = new stdClass();
                    $tempReturnObject->customer = $customerParsed;
                    $tempReturnObject->addresses = $addressesParsed;
                    $tempReturnObject->cart = $respCartDetails;
                    $tempReturnObject->cart->cart_products = $respCartProducts;
                    $tempReturnObject->cart->cart_url = $cartUrl;
                    $returnData[] = $tempReturnObject;
                }
            }
            $custObject = new stdClass();
            $custObject->name = "customers";
            $custObject->description = "shop customers";
            $custObject->object = $returnData;

            $this->eventPayloadData('MassCustomer', $custObject, 'massdata');

            $this->sendDbData($custObject, $uuid);

            if (count($customers) < $limit) {
                $flag = false;
            }
            $offset += $limit;
            $delayCounter++;

            if ($delayCounter == 2) {
                $delayCounter = 0;
                sleep(1);
            }
        }

        $offset = 0;
        $limit = 100;
        $flag = true;
        $this->eventExecutedLog('MassProduct', 'massdata');
        while ($flag) {
            $DBProducts = Db::getInstance()->executeS(
                "SELECT * FROM " . _DB_PREFIX_ . "product ORDER BY id_product LIMIT $limit OFFSET $offset"
            );

            $this->eventGrabData('MassProduct', count($DBProducts), 'massdata');

            $products = array();

            foreach ($DBProducts as $product) {
                $products[] = $this->getProductData($product["id_product"]);
            }

            $prodObject = new stdClass();
            $prodObject->name = "products";
            $prodObject->description = "shop products";
            $prodObject->object = $products;

            $this->eventPayloadData('MassProduct', $prodObject, 'massdata');

            $this->sendDbData($prodObject, $uuid);

            if (count($DBProducts) < $limit) {
                $flag = false;
            }
            $offset += $limit;
            if ($delayCounter == 2) {
                $delayCounter = 0;
                sleep(1);
            }
        }

        $offset = 0;
        $limit = 100;
        $flag = true;
        $this->eventExecutedLog('MassOrder', 'massdata');
        while ($flag) {
            $DBOrders = Db::getInstance()->executeS(
                "SELECT * FROM " . _DB_PREFIX_ . "orders ORDER BY id_order LIMIT $limit OFFSET $offset"
            );

            $this->eventGrabData('MassOrder', count($DBOrders), 'massdata');

            $langId = $this->context->language->id;
            $orders = array();

            foreach ($DBOrders as $DBOrder) {
                $order = new Order($DBOrder["id_order"], $langId);
                $customer = new Customer($order->getCustomer()->id);
                $orderStatus = $order->getCurrentOrderState();
                $address = $customer->getAddresses($langId)[0];
                $orderProducts = $order->getProducts();
                $orderProductsParsed = array();

                foreach ($orderProducts as $product) {
                    $temp = array();
                    $temp["product_id"] = $product["product_id"];
                    $temp["option_id"] = $product["product_attribute_id"];
                    $temp["quantity"] = $product["product_quantity"];
                    $orderProductsParsed[] = $temp;
                }
                $orderData = array();
                $orderData["order_id"] = $order->id;
                $orderData["status"] = $orderStatus->name[$langId];
                $orderData["tracking_id"] = $order->shipping_number;
                $orderData["transport_company"] = $this->getCarrierName($order->id);
                $orderData["shipping_method"] = $this->getCarrierName($order->id);
                $orderData["payment_method"] = $order->payment;
                $orderData["total_price"] = $order->total_paid_tax_incl;
                $orderData["customer_id"] = $order->id_customer;
                $orderData["birthday"] = $customer->birthday;
                $orderData["email"] = $customer->email;
                $orderData["newsletter"] = $customer->newsletter;
                $orderData["website"] = $customer->website;
                $orderData["company"] = $address["company"];
                $orderData["lastname"] = $address["lastname"];
                $orderData["firstname"] = $address["firstname"];
                $orderData["address1"] = $address["address1"];
                $orderData["address2"] = $address["address2"];
                $orderData["postcode"] = $address["postcode"];
                $orderData["city"] = $address["city"];
                $orderData["country"] = $address["country"];
                $orderData["phone"] = $address["phone"];
                $orderData["phone_mobile"] = $address["phone_mobile"];
                $orderData["date_add"] = $address["date_add"];
                $orderData["date_upd"] = $address["date_upd"];
                $orderData["order_products"] = $orderProductsParsed;
                $orders[] = $orderData;
            }
            $orderObject = new stdClass();
            $orderObject->name = "orders";
            $orderObject->description = "shop orders";
            $orderObject->object = $orders;

            $this->eventPayloadData('MassOrder', $orderObject, 'massdata');

            $this->sendDbData($orderObject, $uuid);

            if (count($DBOrders) < $limit) {
                $flag = false;
            }
            $offset += $limit;
            if ($delayCounter == 2) {
                $delayCounter = 0;
                sleep(1);
            }
        }
    }

    /**
     * Get private properties of class
     *
     * @param $obj
     * @param $name
     * @return mixed
     */
    private function getPrivateValue($obj, $name)
    {
        if(empty($obj)){
            return '';
        }

        $r = new ReflectionObject($obj);
        $p = $r->getProperty($name);
        $p->setAccessible(true);
        return $p->getValue($obj);
    }

    /**
     * Get private properties of parent class
     *
     * @param $obj
     * @param $name
     * @return mixed
     */
    private function getParentValue($obj, $name)
    {
        if(empty($obj)){
            return '';
        }
        $r = new ReflectionObject($obj);
        $p = $r->getParentClass()->getProperty($name);
        $p->setAccessible(true);
        return $p->getValue($obj);
    }

    /**
     * Get the UUID from DB
     *
     * @return string
     */
    private function getUUID()
    {
        $uuid = Db::getInstance()->executeS("SELECT value FROM `" . _DB_PREFIX_ . "configuration` WHERE name='uuid';");
        return isset($uuid[0]['value']) ? $uuid[0]['value'] : '';
    }

    /**
     * Get a country name
     *
     * @param $id
     * @return mixed
     */
    private function getCountryById($id)
    {
        $langId = $this->context->language->id;
        $countryResult = Db::getInstance()->executeS(
            "SELECT `name` FROM " . _DB_PREFIX_ . "country_lang WHERE id_country='$id' AND id_lang='$langId'"
        );
        return $countryResult[0]['name'];
    }

    /**
     * Get product options
     *
     * @param $product_id
     * @return array
     */
    public function getProductOptions($product_id)
    {
        $finalOptions = array();
        try {
            $product = new Product($product_id);
			$productOptions = $product->getAttributeCombinations(Context::getContext()->language->id);

            $parsedProductOptions = array();
			if( !empty( $productOptions ) ) {
				foreach ($productOptions as $option) {
					$key = $option["id_product_attribute"];
					$name = Tools::strtolower($option["group_name"]);
					$value = $option["attribute_name"];

					$parsedProductOptions[$key]["option_id"] = $key;
					$parsedProductOptions[$key][$name] = $value;
					$parsedProductOptions[$key]["stock_quantity"] = $option["quantity"];
					$parsedProductOptions[$key]["available_date"] = $option["available_date"];
					$parsedProductOptions[$key]["price_impact"] = $option["price"] * (1 + ($product->getTaxesRate() / 100));
				}
			
				foreach ($parsedProductOptions as $option) {
					$finalOptions[] = $option;
				}
			}
        } catch (PrestaShopDatabaseException $e) {
        } catch (PrestaShopException $e) {
        }

        return $finalOptions;
    }

    /**
     * Get image link of a product
     *
     * @param $id_product
     * @return string
     */
    private function getImgUrl($id_product)
    {
        $imagePath = null;
        try {
            $product = new Product((int)$id_product, false, Context::getContext()->language->id);
            $image = Image::getCover($id_product);
			$id_image = isset($image['id_image']) ? (int)$image['id_image'] : '';
            $link = new Link;

            if (method_exists('ImageType','getFormattedName')) {
                $imageType = ImageType::getFormattedName('home');
            } else {
                $imageType = ImageType::getFormatedName('home');
            }
            
            $imageName = isset($product->link_rewrite) ? $product->link_rewrite : $product->name;
            $imagePath = $link->getImageLink($imageName, $id_image, $imageType);
        } catch (PrestaShopDatabaseException $e) {
        } catch (PrestaShopException $e) {
        }

        return $imagePath;
    }

    /**
     * Get the link of a product
     *
     * @param $productId
     * @return string
     */
    private function getProductUrl($productId)
    {
        $productUrl = null;
        try {
            $link = new Link();
            $product = new Product($productId);
            $productUrl = $link->getProductLink($product);
        } catch (PrestaShopDatabaseException $e) {
        } catch (PrestaShopException $e) {
        }


        return $productUrl;
    }

    /**
     * Get the price of a product without any discounts
     *
     * @param $productId
     * @return float
     */
    private function getProductOriginalPrice($productId)
    {
        $price = null;
        try {
            $product = new Product($productId);
            $price = $product->getPriceWithoutReduct(false, null, 2);
        } catch (PrestaShopDatabaseException $e) {
        } catch (PrestaShopException $e) {
        }

        return $price;
    }

    /**
     * Get carrier of an order
     *
     * @param $id_order
     * @return mixed
     */
    private function getCarrierName($id_order)
    {
        $query = 'SELECT cl.`name` as `carrier_name` FROM `' .
            _DB_PREFIX_ . 'order_carrier` oc LEFT JOIN `' .
            _DB_PREFIX_ . 'carrier` cl ON (oc.`id_carrier` = cl.`id_carrier`) WHERE oc.`id_order` = ' . (int)$id_order;
        $DBCarrier = Db::getInstance()->executeS($query);
        return $DBCarrier[0]["carrier_name"];
    }

    /**
     * Get product custom data
     *
     * @param $productId
     * @param null | $productObject
     * @return array
     */
    private function getProductData($productId, $productObject = null)
    {
        if ($productObject == null) {
            $productObject = new stdClass();

            try {
                $tempProductObject = new Product($productId);
                $productObject->name = $tempProductObject->name;
                $productObject->description = $tempProductObject->name;
                $productObject->description_short = $tempProductObject->name;
                $productObject->quantity = Product::getQuantity($productId);
            } catch (PrestaShopDatabaseException $e) {
            } catch (PrestaShopException $e) {
            }
        }

        $langId = $this->context->language->id;
        $productUrl = $this->getProductUrl($productId);
        $productImage = $this->getImgUrl($productId);
        $price = $this->getProductOriginalPrice($productId);
        $tempCategories = Product::getProductCategoriesFull($productId);
        $categories = array();

        foreach ($tempCategories as $category) {
            $categories[] = $category["name"];
        }

        $specificPrices = SpecificPrice::getByProductId($productId);
        $discounts = array();

		if( !empty( $specificPrices ) ) {
			foreach ($specificPrices as $specificPrice) {
				$discountPrice = "";
				$reductionType = Tools::strtolower($specificPrice["reduction_type"]);

				if ($reductionType == "percentage") {
					$discountPrice = $price - ($price * $specificPrice["reduction"]);
				} elseif ($reductionType == "amount") {
					$reduction = $specificPrice["reduction"];

					if ($specificPrice["reduction_tax"] == false) {
						continue;
					}
					$discountPrice = $price - $reduction;
				}

				$temp_discounts = array(
					"price" => round($discountPrice, 2),
					"date_start" => $specificPrice["from"],
					"date_end" => $specificPrice["to"],
					"quantity" => "",
				);
				$discounts[] = $temp_discounts;
			}
		}

        $p_name = $productObject->name;
        $p_desc = $productObject->description;
        $p_short_desc = $productObject->description_short;

        $data = array(
            "product_id" => $productId,
            "name" => $p_name[$langId] ?? '',
            "description" => $p_desc[$langId] ?? '',
            "short_description" => $p_short_desc[$langId] ?? '',
            "categories" => $categories,
            "stock_quantity" => Product::getQuantity($productId),
            "price" => $price,
            "discounts" => $discounts,
            "image_link" => $productImage,
            "product_link" => $productUrl,
            "product_options" => $this->getProductOptions($productId)
        );

        return $data;
    }

    /**
     * Send mass data
     *
     * @param $data
     * @param $uuid
     */
    private function sendDbData($data, $uuid)
    {
        $curlData = array(
            "data" => array(
                $data
            ),
            "uuid" => $uuid,
        );

        $this->request($this->dataUrl, $curlData, "POST", 'massdata');
    }

    /**
     * CURL Request
     *
     * @param $url
     * @param $data
     * @param $method
     * @return array
     */
    private function request($url, $data, $method, $mode="")
    {
        $event = $data['event'] ?? '';
        try {
            $curl = curl_init();
            curl_setopt_array($curl, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => "",
                CURLOPT_MAXREDIRS => 6,
                CURLOPT_TIMEOUT => 30,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => $method,
                CURLOPT_HTTPHEADER => array("content-type: application/json"),
                CURLOPT_POSTFIELDS => json_encode($data)
            ));

            $curl_response = curl_exec($curl);
            $error = curl_error($curl);
            $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            curl_close($curl);

            $flag = false;
            if (!$error && $code == 200 && !empty($curl_response)) {
                $flag = true;
            }

            $response = array();
            $response["success"] = $flag;
            $response["data"] = array();
            $response["data"]["code"] = $code;
            $response["data"]["error"] = $error;
            $response["response"] = new stdClass();

            if (empty($curl_response)) {
                $response["response"]->message = 'Request timed out. Please find a better connection or try again later.';
            } else {
                $response["response"] = json_decode($curl_response);
            }
            $this->writeLog( $mode, $event, '' );
        } catch (\Exception $exception) {
            $this->writeLog( $mode, $event, $exception );
        }

        return $response;
    }

    /**
     * @param $mode
     * @param $event
     * @param $exception
     * @return void
     */
    private function writeLog( $mode, $event, $exception ) {
        $code = empty($exception) ? '200' : $exception->getCode();
        $msg = empty($exception) ? 'HTTP executed successfully.' : $exception->getMessage();
        $message = array(
            'mode' => $mode,
            'event' => $event,
            'code' => $code,
            'exception_message' => $msg,
            'uuid' => $this->getUUID()
        );
        //Writing http call logs
        $this->routeeWriteLogs($mode, $code, $message);
    }


    /* ============================================ */
    /*                HOOK FUNCTIONS                */
    /* ============================================ */

    /**
     * @param array $params
     * @return void
     */
    public function hookActionCartSave(array $params)
    {
        $this->routeeRemoveLog();
        $this->eventExecutedLog('CartUpdate', 'events');

        if (isset($params['cart']->id) && isset($params['cart']->id_customer)) {
            $id = $params['cart']->id;
            $customerId = $params['cart']->id_customer;

            if ($customerId > 0) {
                $cartUrl = Tools::getHttpHost(true) . __PS_BASE_URI__ . "cart?action=show";
                $data = new stdClass();
                $orders = Db::getInstance()->executeS(
                    "SELECT * FROM " . _DB_PREFIX_ . "orders WHERE id_cart='$id'"
                );

                if (!(is_array($orders) && count($orders) > 0)) {
                    $data->customer_id = $customerId;
                    $data->cart_products = array();
                    $data->cart_url = $cartUrl;

                    $cart_products = Db::getInstance()->executeS(
                        "SELECT * FROM " . _DB_PREFIX_ . "cart_product WHERE id_cart='$id'"
                    );

                    $this->eventGrabData('CartUpdate', $cart_products, 'events');

                    if (is_array($cart_products) && count($cart_products) > 0) {
                        foreach ($cart_products as $cart_product) {
                            $cartPro = new stdClass();
                            $cartPro->product_id = $cart_product['id_product'];
                            $cartPro->quantity = $cart_product['quantity'];
                            $cartPro->option_id = $cart_product['id_product_attribute'];
                            $data->cart_products[] = $cartPro;
                        }

                        $whr = "AND event = 'cartUpdate'";
                        $exists = $this->routeeCheckCart($id, $whr);
                        
                        if(empty($exists)) {
                            $event = "cartUpdate";
                            $qryfor = "insert";

    						$curlData = array(
    							"event" => "CartUpdate",
    							"data" => $data,
    							"uuid" => $this->getUUID()
    						);

                            $this->eventPayloadData('CartUpdate', $curlData, 'events');
    						$this->request($this->eventUrl, $curlData, "POST", 'events');
                            $this->routeeUpdateEventTable($data, $event, $qryfor, $whr, 0, 0, $id);
                        }
                    }
                }
            }
        }
    }

    /**
     * @return void
     */
    public function routeeRemoveLog() {
        $logsTable = _DB_PREFIX_ . "store_events_logs";
        $query = "DELETE FROM $logsTable WHERE datediff(now(), $logsTable.created_at) > 7";

        Db::getInstance()->executeS($query);
    }

    /**
     * @param array $params
     * @return void
     */
    public function hookActionCustomerAccountAdd(array $params)
    {
        $this->eventExecutedLog('CustomerAdd', 'events');

        $customerParsed = array(
            "id" => $params["newCustomer"]->id,
            "lastname" => $params["newCustomer"]->lastname,
            "firstname" => $params["newCustomer"]->firstname,
            "birthday" => $params["newCustomer"]->birthday,
            "email" => $params["newCustomer"]->email,
            "company" => $params["newCustomer"]->company,
            "newsletter" => (bool)$params["newCustomer"]->newsletter,
            "website" => $params["newCustomer"]->website
        );

        $this->eventGrabData('CustomerAdd', $customerParsed, 'events');

        $curlData = array(
            "event" => "CustomerAdd",
            "data" => array(
                "customer" => $customerParsed
            ),
            "uuid" => $this->getUUID()
        );

        $this->eventPayloadData('CustomerAdd', $curlData, 'events');

        $this->request($this->eventUrl, $curlData, "POST", 'events');
    }

    /**
     * @param array $params
     * @return void
     */
    public function hookActionCustomerAccountUpdate(array $params)
    {
        $this->eventExecutedLog('CustomerUpdate', 'events');

        $id = $params["customer"]->id;
        $addresses = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "address WHERE id_customer='$id'");

        $customerParsed = array(
            "id" => $params["customer"]->id,
            "lastname" => $params["customer"]->lastname,
            "firstname" => $params["customer"]->firstname,
            "birthday" => $params["customer"]->birthday,
            "email" => $params["customer"]->email,
            // "company" => $params["customer"]->company,
            "newsletter" => (bool)$params["customer"]->newsletter,
            "website" => $params["customer"]->website
        );

        $this->eventGrabData('CustomerUpdate', $customerParsed, 'events');

        $addressesParsed = array();
        if(isset($addresses)){
            foreach ($addresses as $address) {
                $addressParsed = array(
                    "lastname" => $address["lastname"],
                    "firstname" => $address["firstname"],
                    "address1" => $address["address1"],
                    "address2" => $address["address2"],
                    "postcode" => $address["postcode"],
                    "phone" => $address["phone"],
                    "phone_mobile" => $address["phone_mobile"],
                    "city" => $address["city"],
                    "country" => $this->getCountryById($address["id_country"]),
                    "company" => $address["company"]
                );

                $addressesParsed[] = $addressParsed;
            }
        }
        $this->eventGrabData('CustomerUpdate', $addressesParsed, 'events');

        $curlData = array(
            "event" => "CustomerUpdate",
            "data" => array(
                "customer" => $customerParsed,
                "addresses" => $addressesParsed
            ),
            "uuid" => $this->getUUID()
        );

        $this->eventPayloadData('CustomerUpdate', $curlData, 'events');

        $this->request($this->eventUrl, $curlData, "POST", 'events');
    }

    /**
     * @param array $params
     * @return void
     */
    public function hookActionValidateCustomerAddressForm(array $params)
    {
        $this->eventExecutedLog('CustomerUpdate', 'events');

        $id = $params['cart']->id_customer;
        $customer = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "customer WHERE id_customer='$id'");
        $addresses = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "address WHERE id_customer='$id'");

        $this->eventGrabData('CustomerUpdate', $customer, 'events');


        $formFields = $this->getPrivateValue($params["form"], "formFields");
        $firstname = $this->getParentValue($formFields["firstname"], "value");
        $lastname = $this->getParentValue($formFields["lastname"], "value");
        $company = $this->getParentValue(isset($formFields["company"]) ?? '', "value");
        $address1 = $this->getParentValue($formFields["address1"], "value");
        $address2 = $this->getParentValue($formFields["address2"], "value");
        $postcode = $this->getParentValue($formFields["postcode"], "value");
        $city = $this->getParentValue($formFields["city"], "value");
        $phone = $this->getParentValue($formFields["phone"], "value");
        $countryId = $this->getParentValue($formFields["id_country"], "value");
        $country = $this->getCountryById($countryId);
        $mobile = "";
        try {
            $mobile = $this->getParentValue($formFields["phone"], "value");
        } catch (Exception $e) {
        }

        (isset($mobile)) ? $phone_mobile = $mobile : $phone_mobile = "";

        $customerParsed = array(
            "id" => $customer[0]["id_customer"],
            "lastname" => $customer[0]["lastname"],
            "firstname" => $customer[0]["firstname"],
            "birthday" => $customer[0]["birthday"],
            "email" => $customer[0]["email"],
            // "company" => $customer[0]["company"],
            "newsletter" => (bool)$customer[0]["newsletter"],
            "website" => $customer[0]["website"]
        );

        $addressesParsed = array();

        if(isset($addresses)){
            foreach ($addresses as $address) {
                $addressParsed = array(
                    "lastname" => $address["lastname"],
                    "firstname" => $address["firstname"],
                    "address1" => $address["address1"],
                    "address2" => $address["address2"],
                    "postcode" => $address["postcode"],
                    "phone" => $address["phone"],
                    "phone_mobile" => $address["phone_mobile"],
                    "city" => $address["city"],
                    "country" => $this->getCountryById($address["id_country"]),
                    "company" => $address["company"]
                );
                $addressesParsed[] = $addressParsed;
            }
        }

        $addressesParsed[] = array(
            "firstname" => $firstname,
            "lastname" => $lastname,
            "address1" => $address1,
            "address2" => $address2,
            "postcode" => $postcode,
            "phone" => $phone,
            "phone_mobile" => $phone_mobile,
            "city" => $city,
            "country" => $country,
            "company" => $company
        );

        $this->eventGrabData('CustomerUpdate', $addressesParsed, 'events');

        $curlData = array(
            "event" => "CustomerUpdate",
            "data" => array(
                "customer" => $customerParsed,
                "addresses" => $addressesParsed
            ),
            "uuid" => $this->getUUID()
        );

        $this->eventPayloadData('CustomerUpdate', $curlData, 'events');

        $this->request($this->eventUrl, $curlData, "POST", 'events');
    }

    /**
     * This function will only work for Prestashop 1.6.X
     */
    public function hookActionObjectAddressAddAfter($params) 
    {
        $id_customer = (int)$params['object']->id_customer;
        $this->routeeCustomerUpdateVer16($id_customer);
    }

    /**
     * @param $params
     * @return void
     */
    public function hookActionObjectAddressUpdateAfter($params)
    {
        $id_customer = (int)$params['object']->id_customer;
        $this->routeeCustomerUpdateVer16($id_customer);
    }

    /**
     * @param $params
     * @return void
     */
    public function hookActionObjectCustomerUpdateAfter($params)
    {
        $id_customer = (int)$params['object']->id;
        $this->routeeCustomerUpdateVer16($id_customer);
    }

    /**
     * @param $id
     * @return void
     */
    public function routeeCustomerUpdateVer16($id)
    {
        $this->eventExecutedLog('CustomerUpdate', 'events');

        $customer = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "customer WHERE id_customer='$id'");
        $addresses = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "address WHERE id_customer='$id'");

        $customerParsed = array(
            "id" => $customer[0]["id_customer"],
            "lastname" => $customer[0]["lastname"],
            "firstname" => $customer[0]["firstname"],
            "birthday" => $customer[0]["birthday"],
            "email" => $customer[0]["email"],
            "newsletter" => (bool)$customer[0]["newsletter"],
            "website" => $customer[0]["website"]
        );

        $this->eventGrabData('CustomerUpdate', $customerParsed, 'events');

        $addressesParsed = array();

        if(isset($addresses)){
            foreach ($addresses as $address) {
                $addressParsed = array(
                    "lastname" => $address["lastname"],
                    "firstname" => $address["firstname"],
                    "address1" => $address["address1"],
                    "address2" => $address["address2"],
                    "postcode" => $address["postcode"],
                    "phone" => $address["phone"],
                    "phone_mobile" => $address["phone_mobile"],
                    "city" => $address["city"],
                    "country" => $this->getCountryById($address["id_country"]),
                    "company" => $address["company"]
                );
                $addressesParsed[] = $addressParsed;
            }
        }

        $this->eventGrabData('CustomerUpdate', $addressesParsed, 'events');

        $curlData = array(
            "event" => "CustomerUpdate",
            "data" => array(
                "customer" => $customerParsed,
                "addresses" => $addressesParsed
            ),
            "uuid" => $this->getUUID()
        );

        $this->eventPayloadData('CustomerUpdate', $curlData, 'events');

        $this->request($this->eventUrl, $curlData, "POST", 'events');
    }

    /**
     * @param $message
     * @param $level
     * @param $fileName
     * @return false|int
     */
    public static function log($message, $level = 'debug', $fileName = null){
        $fileDir = _PS_ROOT_DIR_ . '/log/';
        $fileName=self::DEFAULT_LOG_FILE;
        if(is_array($message) || is_object($message)){$message = print_r($message, true);}
        $formatted_message=$level." -- ".date('Y/m/d - H:i:s').": ".$message."\r\n";
        return file_put_contents($fileDir . $fileName, $formatted_message, FILE_APPEND);
    }


    /**
     * @param array $params
     * @return void
     */
    public function hookActionOrderStatusPostUpdate(array $params)
    {
        $this->eventExecutedLog('OrderStatusUpdate', 'events');

        // Load order object
        $order = new Order((int) $params["id_order"]);
        // Validate customer object
        if (Validate::isLoadedObject($order)) {
            $data = array(
                "customer_id" => $order->id_customer,
                "order_id" => $params["id_order"],
                "order_status" => $params["newOrderStatus"]->name,
                'tracking_id' => $order->shipping_number,
                'transport_company' => $this->getCarrierName( $params["id_order"] )
            );
            $this->eventGrabData('OrderStatusUpdate', $data, 'events');

            $curlData = array(
                "event" => "OrderStatusUpdate",
                "data" => $data,
                "uuid" => $this->getUUID()
            );

            $this->eventPayloadData('OrderStatusUpdate', $curlData, 'events');

            $this->request($this->eventUrl, $curlData, "POST", 'events');
        }
    }

    /**
     * @param array $params
     * @return void
     */
    public function hookActionValidateOrder(array $params)
    {
        $this->eventExecutedLog('OrderAdd', 'events');

        $addresses = Db::getInstance()->executeS(
            "SELECT * FROM " . _DB_PREFIX_ . "address WHERE id_customer='" . $params["customer"]->id . "'"
        );

        $order_details = array(
            "order_id" => $params["order"]->id,
            "order_status" => $params["orderStatus"]->name,
            "shipping_method" => $this->getCarrierName($params["order"]->id),
            "payment_method" => $params["order"]->payment,
            "total_price" => $params["order"]->total_paid,
            "customer_id" => $params["customer"]->id,
            "lastname" => $params["customer"]->lastname,
            "firstname" => $params["customer"]->firstname,
            "birthday" => $params["customer"]->birthday,
            "email" => $params["customer"]->email,
            "company" => $params["customer"]->company,
            "newsletter" => (bool)$params["customer"]->newsletter,
            "website" => $params["customer"]->website,
            "address1" => $addresses[0]["address1"],
            "address2" => $addresses[0]["address2"],
            "postcode" => $addresses[0]["postcode"],
            "phone" => $addresses[0]["phone"],
            "phone_mobile" => $addresses[0]["phone_mobile"],
            "city" => $addresses[0]["city"],
            "country" => $this->getCountryById($addresses[0]["id_country"])
        );
        $this->eventGrabData('OrderAdd', $order_details, 'events');

        $order_products = array();

        foreach ($params["order"]->product_list as $product) {
            $temp_product = array();
            $temp_product["product_id"] = $product["id_product"];
            $temp_product["quantity"] = $product["cart_quantity"];
            $temp_product["option_id"] = $product["id_product_attribute"];
            $temp_product["price"] = $product["price_without_reduction"];
            $temp_product["discounted_price"] = $product["price"];
            $order_products[] = $temp_product;
        }

        $this->eventGrabData('OrderAdd', $order_products, 'events');

        $data = array(
            "customer_id" => $params["order"]->id_customer,
            "order_details" => $order_details,
            "order_products" => $order_products
        );

        $curlData = array(
            "event" => "OrderAdd",
            "data" => $data,
            "uuid" => $this->getUUID()
        );

        $this->eventPayloadData('OrderAdd', $curlData, 'events');

        $this->request($this->eventUrl, $curlData, "POST", 'events');
    }

    /**
     * @param array $params
     * @return void
     */
    public function hookActionPaymentConfirmation(array $params)
    {
        $this->eventExecutedLog('OrderPaymentConfirmed', 'events');

        $data = array(
            "order_id" => $params["id_order"],
            "customer_id" => $params["cart"]->id_customer
        );

        $this->eventGrabData('OrderPaymentConfirmed', $data, 'events');

        $curlData = array(
            "event" => "OrderPaymentConfirmed",
            "data" => $data,
            "uuid" => $this->getUUID()
        );

        $this->eventPayloadData('OrderPaymentConfirmed', $curlData, 'events');

        $this->request($this->eventUrl, $curlData, "POST", 'events');
    }

    /**
     * @param array $params
     * @return void
     */
    public function hookActionProductSave(array $params)
    {
        $this->eventExecutedLog('ProductAdd/Update', 'events');

        $event = $qryfor = $whr = "";
        $data = $this->getProductData($params["id_product"], $params["product"]);
        if(!empty($data['name']) || !empty($data['name']) || $data['price'] > 0) {
            
            $whr = "AND event != 'ProductStockUpdate'";
            $exists = $this->routeeCheckProduct($data['product_id'], $whr);

            if(empty($exists)) {
                $event = "ProductAdd";
                $qryfor = "insert";
            } else{
                if(isset($exists[0]['json'])) {
                    $compare = (json_decode($exists[0]['json'], true) == $data);
                }

                if(!$compare){
                    $pid = $data['product_id'];
                    $event = "ProductUpdate";
                    $qryfor = "update";
                    $whr = "product_id = '$pid' $whr";
                }
            }

            $this->eventGrabData($event, $data, 'events');

            if(!empty($event)){
                $curlData = array(
                    "data" => $data,
                    "event" => $event,//"ProductAdd",
                    "uuid" => $this->getUUID()
                );

                $this->eventPayloadData($event, $curlData, 'events');

                $this->request($this->eventUrl, $curlData, "POST", 'events');

                $this->routeeUpdateEventTable($data, $event, $qryfor, $whr, $data['product_id'], 0, 0);    
            }
        }
    }

    /**
     * @param array $params
     * @return void
     */
    public function hookActionProductUpdate(array $params)
    {
        $this->eventExecutedLog('ProductUpdate', 'events');

        $data = $this->getProductData($params["id_product"], $params["product"]);
        $this->eventGrabData('ProductUpdate', $data, 'events');

        $curlData = array(
            "data" => $data,
            "event" => "ProductUpdate",
            "uuid" => $this->getUUID()
        );

        $this->eventPayloadData('ProductUpdate', $curlData, 'events');

        $this->request($this->eventUrl, $curlData, "POST", 'events');
    }

    /**
     * @param array $params
     * @return void
     */
    public function hookActionProductOutOfStock(array $params)
    {
        $this->eventExecutedLog('ProductOutOfStock', 'events');

        $id = $params["product"]->id;
        $qty = $params["product"]->quantity;

        $this->eventGrabData('ProductOutOfStock', ['id' => $id, 'qty' => $qty], 'events');

        if (is_numeric($id) && $qty < 1) {
            $curlData = array(
                "event" => "ProductOutOfStock",
                "data" => array(
                    "product_id" => $id
                ),
                "uuid" => $this->getUUID()
            );

            $whr = "AND event = 'ProductOutOfStock'";
            $exists = $this->routeeCheckProduct($id, $whr);
            
            if(empty($exists)) {
                $event = "ProductOutOfStock";
                $qryfor = "insert";

                $this->eventPayloadData('ProductOutOfStock', $curlData, 'events');

                $this->request($this->eventUrl, $curlData, "POST", 'events');
                $this->routeeUpdateEventTable($curlData['data'], $event, $qryfor, $whr, $id, 0, 0);
            }            
        }
    }

    /**
     * @param array $params
     * @return void
     */
    public function hookActionUpdateQuantity(array $params)
    {
        $this->eventExecutedLog('ProductStockUpdate', 'events');

        $productId = $params["id_product"];
        $quantity = $params["quantity"];
        $optionId = $params["id_product_attribute"];

        $this->eventGrabData('ProductStockUpdate', ['id' => $productId, 'qty' => $quantity, 'optionId' => $optionId], 'events');

        $curlData = array(
            "event" => "ProductStockUpdate",
            "data" => array(
                "product_id" => $productId,
                "quantity" => $quantity,
                "option_id" => $optionId
            ),
            "uuid" => $this->getUUID());

        $event = $qryfor = "";
        $whr = "AND event = 'ProductStockUpdate'";
        $exists = $this->routeeCheckProduct($productId, $whr);
        
        if(empty($exists)) {
            $event = "ProductStockUpdate";
            $qryfor = "insert";
        } else {
            if(isset($exists[0]['json'])) {
                $compare = (json_decode($exists[0]['json'], true) === $curlData['data']);
            }

            if(!$compare){
                $pid = $productId;
                $event = "ProductStockUpdate";
                $qryfor = "update";
                $whr = "product_id = '$productId' AND event = 'ProductStockUpdate'";
            }
        }

        if(!empty($event)){
            $this->eventPayloadData('ProductStockUpdate', $curlData, 'events');

            $this->request($this->eventUrl, $curlData, "POST", 'events');
            $this->routeeUpdateEventTable($curlData['data'], $event, $qryfor, $whr, $productId, 0, 0);    
        }
    }

    /**
     * @param array $params
     * @return void
     */
    public function hookActionProductDelete(array $params)
    {
    }

    /**
     * @param array $params
     * @return void
     */
    public function hookActionSearch(array $params)
    {
        $this->eventExecutedLog('Search', 'events');

        $params["searched_query"];

		$custId = $params["cart"]->id_customer;
		$customer = [];
		if($custId > 0){
			$customer = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "customer WHERE id_customer='$custId'");
		}

        $data = array(
            "search_string" => $params["searched_query"],
			"customer_id" => $custId ?? 0,
            "email" => $customer[0]["email"] ?? '',
            "phone" => '',//Not available in prestashop
        );

        $this->eventGrabData('Search', $data, 'events');

        $whr = "AND event = 'Search'";
        $id = ($params["cart"]->id_customer > 0) ? $params["cart"]->id_customer : $params["cart"]->id_guest;
        $exists = $this->routeeCheckSearch($id, $whr);
        
        if(empty($exists)) {
            $event = "Search";
            $qryfor = "insert";

            $curlData = array(
                "event" => "Search",
                "data" => $data,
                "uuid" => $this->getUUID()
            );

            $this->eventPayloadData('Search', $curlData, 'events');

            $this->request($this->eventUrl, $curlData, "POST", 'events');
            $this->routeeUpdateEventTable($data, $event, $qryfor, $whr, 0, $id, 0);
        }
    }

    /**
     * @param $product_id
     * @param $whr
     * @return mixed
     */
    public function routeeCheckProduct($product_id, $whr)
    {
        $get_product = Db::getInstance()->executeS(
                "SELECT json FROM `" . _DB_PREFIX_ . "routee_events` WHERE product_id='".$product_id."' ".$whr.";"
            );
        return $get_product;
    }

    /**
     * @param $cartId
     * @param $whr
     * @return mixed
     */
    public function routeeCheckCart($cartId, $whr)
    {
        $get_cart = Db::getInstance()->executeS(
                "SELECT json FROM `" . _DB_PREFIX_ . "routee_events` WHERE order_id='".$cartId."' ".$whr.";"
            );
        return $get_cart;
    }

    /**
     * @param $customerId
     * @param $whr
     * @return mixed
     */
    public function routeeCheckSearch($customerId, $whr)
    {
        $get_search = Db::getInstance()->executeS(
                "SELECT json FROM `" . _DB_PREFIX_ . "routee_events` WHERE customer_id='".$customerId."' ".$whr.";"
            );
        return $get_search;
    }

    /**
     * @param $data
     * @param $event
     * @param $qryfor
     * @param $whr
     * @param $product_id
     * @param $customer_id
     * @param $order_id
     * @return void
     */
    public function routeeUpdateEventTable($data, $event, $qryfor, $whr, $product_id, $customer_id, $order_id)
    {
        $datajson = json_encode($data);
        if($qryfor == 'insert') {
            Db::getInstance()->execute(
                "INSERT INTO `" . _DB_PREFIX_ . "routee_events` (customer_id, product_id, order_id, event, json) VALUES ('$customer_id', '$product_id', '$order_id', '$event', '$datajson');"
            );
        } else {
            Db::getInstance()->execute(
                "UPDATE `" . _DB_PREFIX_ . "routee_events`
                SET customer_id='$customer_id', product_id='$product_id', order_id='$order_id', event='$event', json='$datajson'
                    WHERE $whr;"
            );
        }
    }

    /**
     * @param $mode
     * @param $code
     * @param $logdata
     * @return void
     */
    function routeeWriteLogs($mode, $code, $logdata) {
        include_once "RouteeLogs.php";
        new RouteeLogs($mode, $code, $logdata);
    }

    /**
     * @param $func
     * @param $method
     * @return void
     */
    function eventExecutedLog($func, $method) {
        $payload = array(
            'function' => $func,
            'method' => $method,
            'desc' => 'Hook function executed.'
        );
        $this->routeeWriteLogs($method, 200, $payload);
    }

    /**
     * @param $func
     * @param $data
     * @param $method
     * @return void
     */
    function eventGrabData($func, $data, $method) {
        $payload = array(
            'function' => $func,
            'method' => $method,
            'postdata' => $data,
            'desc' => 'Event data grabbed.'
        );
        $this->routeeWriteLogs($method, 200, $payload);
    }

    /**
     * @param $func
     * @param $data
     * @param $method
     * @return void
     */
    function eventPayloadData($func, $data, $method) {
        $payload = array(
            'function' => $func,
            'method' => $method,
            'api_payload' => $data,
            'desc' => 'Event API payload is prepared.'
        );
        $this->routeeWriteLogs($method, 200, $payload);
    }

}
