<?php
include_once(dirname(__FILE__).'../../../config/config.inc.php');
include_once(dirname(__FILE__).'../../../init.php');

switch (Tools::getValue('method')) {
    case 'exportCsv' :
        handleLogsCsv();
        $return = array('success' => 'yes', 'data' => handleLogsCsv());
        echo json_encode($return);
        break;
    case 'exportLogApi' :
        $response = handleLogsApi();
        $return = array('success' => 'yes', 'data' => $response);
        echo json_encode($return);
        break;
    default:
        exit;
}

/**
 * @return mixed
 */
function handleLogsCsv()
{
    return Db::getInstance()->executeS(
        "SELECT * FROM " . _DB_PREFIX_ . "store_events_logs WHERE is_exported='0'"
    );
}

/**
 * @return array
 */
function handleLogsApi()
{
    $apiUrl = 'https://autoforms.routee.net/u001/log.php?multi=1';
    $uuid = Db::getInstance()->executeS("SELECT value FROM `" . _DB_PREFIX_ . "configuration` WHERE name='uuid';");
    $uuid = $uuid[0]['value'] ?? '';


    $offset = $delayCounter = 0;
    $limit = 100;
    $flag = true;
    $postArr = $ids = [];
    while ($flag) {
        $logs = Db::getInstance()->executeS(
            "SELECT * FROM " . _DB_PREFIX_ . "store_events_logs WHERE is_exported='0' LIMIT $limit OFFSET $offset"
        );

        foreach ($logs as $log) {
            $ids[] = $log['id'];
            $postArr[] = array(
                'siteUrl' => $log['store_url'],
                'uuid' => $uuid,
                'event_name' => eventName($log['event_type']),
                'log_type' => $log['log_type'] == 1 ? 'success' : 'failure',
                'log_data' => $log['log_data'],
                'created_at' => $log['created_at'],
                'platform' => "Prestashop"
            );
        }

        if (count($logs) < $limit) {
            $flag = false;
        }
        $offset += $limit;
        if ($delayCounter == 2) {
            $delayCounter = 0;
            sleep(1);
        }
    }

    $response = logRequest($apiUrl, $postArr, 'POST');
    routeeUpdateLogs($response['code'], $ids);
    return $response;
}

/**
 * @param $url
 * @param $data
 * @param $method
 * @return array
 */
function logRequest($url, $data, $method)
{
    $curl = curl_init();
    curl_setopt_array($curl, array(
        CURLOPT_URL => $url,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 6,
        CURLOPT_TIMEOUT => 30,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => $method,
        CURLOPT_HTTPHEADER => array("content-type: application/json"),
        CURLOPT_POSTFIELDS => json_encode($data)
    ));

    $curl_response = curl_exec($curl);
    $error = curl_error($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);

    $response = array();
    $response["code"] = $code;
    $response["error"] = $error;
    $response["response"] = json_decode($curl_response);
    return $response;
}

/**
 * @param $eventType
 * @return string
 */
function eventName($eventType) {
    $name = '';

    switch ($eventType) {
        case 1:
            $name = "auth";
            break;
        case 3:
            $name = "massdata";
            break;
        case 2:
            $name = "events";
            break;
    }

    return $name;
}

/**
 * @param $code
 * @return void
 */
function routeeUpdateLogs($code, $ids) {
    if ($code == 200) {
        Db::getInstance()->executeS(
            "UPDATE " . _DB_PREFIX_ . "store_events_logs SET is_exported=1 WHERE is_exported=0 AND id IN (".implode(',',$ids).")"
        );
    }
}