{*
* AMD Telecom
*
* DISCLAIMER
*
* AMD Telecom grants you a revocable, non-exclusive, non-transferable, limited
* license to download, install and use the Router Application solely for your commercial
* purposes strictly in accordance with the terms of this Agreement.
*
* @author AMD Telecom <products@amdtelecom.net>
* @copyright  AMD Telecom SA
* @license https://go.routee.net
*}

{extends file="helpers/form/form.tpl"}
{block name="field"}
    {if $input.type == 'title'}
        <h2 style="text-align: center">{$input.text}</h2>
    {elseif $input.type == 'paragraph'}
        <p style="text-align: center">{$input.text}</p>
    {elseif $input.type == 'username'}
        <div class="col-lg-8">
            <div class="input-group">
                <span class="input-group-addon">
                    <i class="icon-user"></i>
                </span>
                <input type="text"
                       id={$input.name|escape:'html'}
                       name={$input.name|escape:'html'}
                       class="" value="{$input.value|escape:'html'}" required="required">
            </div>
        </div>
    {elseif $input.type == 'password'}
        <div class="col-lg-8">
            <div class="input-group">
                <span class="input-group-addon">
                    <i class="icon-key"></i>
                </span>
                <input type="password"
                       id={$input.name|escape:'html'}
                       name={$input.name|escape:'html'}
                       class="" value="{$input.value|escape:'html'}" required="required">
            </div>
        </div>
    {elseif $input.type == 'routee_uuid' && $input.value != ''}
        <div class="col-lg-8">
            <div class="input-group">
                <span class="input-group-addon">
                    <i class="icon-pencil"></i>
                </span>
                <input type="text"
                       id={$input.name|escape:'html'}
                       name={$input.name|escape:'html'}
                       class="" value="{$input.value|escape:'html'}" required="required" {if $input.value != ''}disabled="disabled"{/if}>
            </div>
        </div>

    {elseif $input.type == 'log_export_csv'}
        <div class="col-lg-8">
            <div class="input-group">
                <button type="button"
                       id="log_export_csv"
                       name={$input.name|escape:'html'}
                       class="">
                    {$input.label|escape:'html'}
                </button>
            </div>
        </div>

    {elseif $input.type == 'log_export_api'}
        <div class="col-lg-8">
            <div class="input-group">
                <button type="button"
                        id="log_export_api"
                        name={$input.name|escape:'html'}
                        class="">
                    {$input.label|escape:'html'}
                </button>
            </div>
        </div>

    {else}
        {$smarty.block.parent}
    {/if}


    <script type="text/javascript">
        $(document).ready(function(){
            //Download Log CSV file
            $("#log_export_csv").unbind().click(function (event) {
                $(this).attr("disabled", "true");
                make_ajax('logs', 'exportCsv');
            })

            //API Log to send data
            $("#log_export_api").unbind().click(function (event) {
                $(this).attr("disabled", "true");
                make_ajax('logsapi', 'exportLogApi');
            })


            /**
             *
             * @param data
             * @param type
             */
            function make_ajax(type, method) {
                $.ajax({
                    type: 'POST',
                    cache: false,
                    dataType: 'json',
                    url: baseDir + 'modules/routee/routee-ajax.php',
                    data: 'method=' + method,
                }).fail(function (jqXHR, textStatus, errorThrown) {
                    console.log('error ' + textStatus);
                }).done(function (data, textStatus, jqXHR) {
                    if(type == 'logs') {
                        generate_csv(data)
                    }

                    if(type == 'logsapi') {
                        logs_api_result(data)
                    }
                });
            }

            /**
             *
             * @param success
             */
            function generate_csv(success) {
                if (success.success == "yes") {
                    var rec_arr = success.data;

                    let csv_arr = [];
                    let csv_header_arr = [
                        "ID",
                        "Store Url",
                        "Log Type",
                        "Event Type",
                        "Log Date",
                        "Is Exported",
                        "Created at"
                    ];
                    csv_arr.push(csv_header_arr);
                    let record_arr = [];

                    for (i = 0; i < rec_arr.length; i++) {

                        record_arr.push(rec_arr[i].id);
                        record_arr.push(rec_arr[i].store_url);
                        record_arr.push(rec_arr[i].log_type);
                        record_arr.push(rec_arr[i].event_type);
                        record_arr.push(rec_arr[i].log_data);
                        record_arr.push(rec_arr[i].is_exported);
                        record_arr.push(rec_arr[i].created_at);
                        csv_arr.push(record_arr);
                        record_arr = [];
                    }

                    var csv_content = arrayToCsv(csv_arr);
                    var timeElapsed = Date.now();
                    var today = new Date(timeElapsed);
                    var year = today.getFullYear();
                    var month = today.getMonth() + 1;
                    var day = today.getDate();
                    var filename = "Routee_Log_" + year + month + day + ".csv";
                    downloadBlob(csv_content, filename, "text/csv;charset=utf-8;");
                }
            }

            /**
             *
             * @param content
             * @param filename
             * @param contentType
             */
            function downloadBlob(content, filename, contentType) {
                // Create a blob
                var blob = new Blob([content], { type: contentType });
                var url = URL.createObjectURL(blob);

                // Create a link to download it
                var pom = document.createElement("a");
                pom.href = url;
                pom.setAttribute("download", filename);
                pom.click();
                pom.remove();
            }

            /**
             *
             * @param data
             * @returns {*}
             */
            function arrayToCsv(data) {
                return data
                    .map(
                        (row) =>
                            row
                                .map(String) // convert every value to String
                                .map((v) => v.replaceAll('"', '""')) // escape double colons
                                .map((v) => `"${ v }"`) // quote it
                                .join(",") // comma-separated
                    )
                    .join("\r\n"); // rows starting on new lines
            }

            /**
             *
             * @param success
             */
            function logs_api_result(success) {
                if(success.data.code == '200') {
                    $("#log_export_api").after('<p><b style="color: green">Logs exported to API successfully.</b></p>');
                } else {
                    $("#log_export_api").after('<p><b style="color: orangered">Logs cannot be exported to API, please try later.</b></p>');
                }
            }
        });

    </script>
{/block}